from cocos.sprite import Sprite
import pyglet
import math
import random
from cocos.actions import MoveBy


class Player(Sprite):
    def __init__(self, world_width, world_height):
        super(Player, self).__init__('warrior_down1.png')
        self.direction = 'down'                             #Initial direction
        self.animations = {}
        for k in ['left', 'down', 'right', 'up']:
            anims = []
            for i in range(4):
                anims.append(pyglet.resource.image('warrior_%s%d.png' % (k, i)))
            self.animations[k] = anims

        #Keep track of the current frame for each direction
        self.current_frames = {'left' : 0, 
                               'down' : 0, 
                               'right': 0, 
                               'up'   : 0}

        self.image = self.animations[self.direction][0]     #Initial image

        self.shadow = Sprite('shadow.png')
        self.shadow.position = (self.position[0], self.position[1] - self.height/2 - 2)
        self.shadow.scale = 0.3

        self.position = (world_width/2, 0)
        self.scale = 0.4
        self.speed = 100

    def set_direction(self, new_dir, dt):
        self.direction = new_dir
        #Update the current frame for the current direction
        self.current_frames[self.direction] += 5 * dt
        #Wrap the current frame number between 0..number_of_frames - 1
        self.current_frames[self.direction] = self.current_frames[self.direction] % len(self.animations[self.direction])
        #print "%s: %s" % (self.direction, self.current_frames[self.direction])
        self.image = self.animations[self.direction][int(math.floor(self.current_frames[self.direction]))]

    def collides(self, a_sprite):
        return not ((self.x > a_sprite.x + a_sprite.width)
            or (self.x + self.width < a_sprite.x)
            or (self.y + self.height < a_sprite.y)
            or (self.y > a_sprite.y + a_sprite.height))


##################################################################################################

class Enemy(Sprite):
    """ Generic enemy."""
    def __init__(self, world_width, world_height, images_left, images_right):
        super(Enemy, self).__init__('thf1_rt1.png')

        images_left  = [pyglet.resource.image(img) for img in images_left]
        images_right = [pyglet.resource.image(img) for img in images_right]
        self.animations = { 'left'  : images_left, 
                            'right' : images_right}

        self.position = (random.randint(0, world_width-300), random.randint(world_height/3, world_height-1))
        self.direction = random.choice(['left', 'right'])

        #Keep track of the current frame for each direction
        self.current_frames = {'left' : 0, 
                               'right': 0}

        self.image = self.animations[self.direction][0]     #Initial image

        if self.direction == 'right':
            radar_pos = (self.position[0]+10, self.position[1])
        elif self.direction == 'left':
            radar_pos = (self.position[0]-10, self.position[1])

        self.radar = Radar(self.direction, radar_pos)

        self.checkpoints = []
        self.checkpoints.append(self.position)
        nuevox, nuevoy = self.position
        nuevox += random.randint(50, 300)
        self.checkpoints.append((nuevox, nuevoy))
        self.next_checkpoint = 0

    def patrol(self, dt):
        if self.arrived(self.checkpoints[self.next_checkpoint]):
            #Turn to the next checkpoint
            self.next_checkpoint = (self.next_checkpoint +1) % len(self.checkpoints)
        destx, desty = self.checkpoints[self.next_checkpoint]
        posx, posy   = self.position
        if posx < destx:
            self.set_direction('right', dt)
            self.do(MoveBy((2, 0), 3*dt))
        else:
            self.set_direction('left', dt)
            self.do(MoveBy((-2, 0), 3*dt))

        #Update the position of the radar
        if self.direction == 'right':
            radar_pos = (self.position[0]+10, self.position[1])
        elif self.direction == 'left':
            radar_pos = (self.position[0]-10, self.position[1])
        self.radar.position = radar_pos

    #Check if the enemy arrived to his destination
    def arrived(self, destination):
        posx, posy   = self.position
        destx, desty = destination
        return (math.floor(posx) == math.floor(destx)) and (math.floor(posy) == math.floor(desty))


    def set_direction(self, new_dir, dt):
        self.direction = new_dir
        #Update the current frame for the current direction
        self.current_frames[self.direction] += 5 * dt
        #Wrap the current frame number between 0..number_of_frames - 1
        self.current_frames[self.direction] = self.current_frames[self.direction] % len(self.animations[self.direction])
        #print "%s: %s" % (self.direction, self.current_frames[self.direction])
        self.image = self.animations[self.direction][int(math.floor(self.current_frames[self.direction]))]

        self.radar.set_direction(self.direction)


##################################################################################################
class Enemy1(Enemy):
    def __init__(self, world_width, world_height):
        super(Enemy1, self).__init__(world_width, world_height, ['thf1_lf1.png', 'thf1_lf2.png'], ['thf1_rt1.png', 'thf1_rt2.png'])

##################################################################################################

class Enemy2(Enemy):
    def __init__(self, world_width, world_height):
        super(Enemy2, self).__init__(world_width, world_height, ['npc6_lf1.png', 'npc6_lf2.png'], ['npc6_rt1.png', 'npc6_rt2.png'])

##################################################################################################

class Radar(Sprite):
    def __init__(self, direction, position):
        if direction == 'left':
            super(Radar, self).__init__('radar-largo-left.png')
        else:
            super(Radar, self).__init__('radar-largo-right.png')
        self.animations = { 'left'  :[pyglet.resource.image('radar-largo-left.png')],
                            'right' :[pyglet.resource.image('radar-largo-right.png')]}
        #Make the radar translucent
        self.opacity = 128

        self.position = position
        self.set_direction(direction)

    def set_direction(self, new_dir):
        self.direction = new_dir
        self.image = self.animations[self.direction][0]

        if self.direction == 'right':
            #Set the image anchor to the vertex that points to the character
            self.image_anchor_x = 0
            self.image_anchor_y = self.height/2

            #Calculate the position of the vertices
            self.vert1 = (self.position[0], self.position[1])
            self.vert2 = (self.position[0]+self.width, self.position[1] +self.height/2)
            self.vert3 = (self.position[0]+self.width, self.position[1]-self.height/2)

        elif self.direction == 'left':
            #Set the image anchor to the vertex that points to the character
            self.image_anchor_x = self.width
            self.image_anchor_y = self.height/2

            #Calculate the position of the vertices
            self.vert1 = (self.position[0], self.position[1])
            self.vert2 = (self.position[0]-self.width, self.position[1] +self.height/2)
            self.vert3 = (self.position[0]-self.width, self.position[1]-self.height/2)


    def get_vector_from_points(self, puntoorigen, puntodestino):
        """ Calculate the vector between two points."""
        return (puntodestino[0]-puntoorigen[0], puntodestino[1]-puntoorigen[1])

    def magnitud_vector(self, vec):
        """ Calculate the magnitude of the vector."""
        return math.sqrt(vec[0]**2 + vec[1]**2)

    def dot_product(self, a, b):
        """ Calculate the dot product between two vectors."""
        return sum([a[i]*b[i] for i in range(len(a))])


    def is_point_inside(self, point):
        """ Decide if a point is inside the radar triangle or not."""
        vector1 = self.get_vector_from_points(point, self.vert1)
        vector2 = self.get_vector_from_points(point, self.vert2)
        vector3 = self.get_vector_from_points(point, self.vert3)

        suma_acos = 0.0
        dp = self.dot_product(vector1, vector2)
        coseno = dp / (self.magnitud_vector(vector1) * self.magnitud_vector(vector2))
        arcocoseno = math.acos(coseno)
        suma_acos += arcocoseno

        dp = self.dot_product(vector2, vector3)
        coseno = dp / (self.magnitud_vector(vector2) * self.magnitud_vector(vector3))
        arcocoseno = math.acos(coseno)
        suma_acos += arcocoseno

        dp = self.dot_product(vector3, vector1)
        coseno = dp / (self.magnitud_vector(vector3) * self.magnitud_vector(vector1))
        arcocoseno = math.acos(coseno)
        suma_acos += arcocoseno

        if math.fabs(suma_acos - 2 * math.pi) < 0.01:                   #arbitrary epsilon = 0.01
            return True
        else:
            return False

##################################################################################################

class King(Sprite):

    def __init__(self, world_width, world_height):
        super(King, self).__init__('king_l1.png')

        self.animations = {'left'  :[pyglet.resource.image('king_l1.png'), pyglet.resource.image('king_l2.png')], 
                            'right':[pyglet.resource.image('king_r1.png'), pyglet.resource.image('king_r2.png')], 
                            'up'   :[pyglet.resource.image('king_u1.png'), pyglet.resource.image('king_u2.png')], 
                            'down' :[pyglet.resource.image('king_d1.png'), pyglet.resource.image('king_d2.png')], 
                            }

        #self.position = (random.randint(0, world_width-self.width-1), random.randint(world_height-200, world_height-1))
        #self.position = (462, 986)
        self.position  = (random.randint(0, world_width-self.width-1), random.randint(world_height*3/4, world_height-1))
        self.direction = 'left'

        #Keep track of the current frame for each direction
        self.current_frames = {'left' : 0, 
                               'right': 0, 
                               'up'   : 0, 
                               'down' : 0}

        self.image = self.animations[self.direction][0]     #Initial image
        self.is_following = False

    def follow(self, position, direction, dt):
        new_pos_x, new_pos_y = position
        if direction == 'up':
            new_pos_y -= 40
        elif direction == 'down':
            new_pos_y += 40
        elif direction == 'right':
            new_pos_x -= 40
        elif direction == 'left':
            new_pos_x += 40

        self.position = (new_pos_x, new_pos_y)
        self.direction = direction

        #Update the current frame for the current direction
        self.current_frames[self.direction] += 5 * dt
        #Wrap the current frame number between 0..number_of_frames - 1
        self.current_frames[self.direction] = self.current_frames[self.direction] % len(self.animations[self.direction])
        #print "%s: %s" % (self.direction, self.current_frames[self.direction])
        self.image = self.animations[self.direction][int(math.floor(self.current_frames[self.direction]))]


    def unfollow(self):
        new_pos_x, new_pos_y = self.position
        if self.direction == 'up':
            new_pos_y -= 40
        elif self.direction == 'down':
            new_pos_y += 40
        elif self.direction == 'right':
            new_pos_x -= 40
        elif self.direction == 'left':
            new_pos_x += 40

        self.position = (new_pos_x, new_pos_y)


    def collides(self, a_sprite):
        return not ((self.x > a_sprite.x + a_sprite.width)
            or (self.x + self.width < a_sprite.x)
            or (self.y + self.height < a_sprite.y)
            or (self.y > a_sprite.y + a_sprite.height))
